@@ -7,16 +7,17 @@ use Mojo::IOLoop::Delay;
 use Mojo::IOLoop::Stream;
 use Mojo::SMTP::Client::Response;
 use Mojo::SMTP::Client::Exception;
-use Carp;
 use Scalar::Util 'weaken';
+use Carp;
 
-our $VERSION = 0.06;
+our $VERSION = 0.07;
 
 use constant {
 	CMD_OK       => 2,
 	CMD_MORE     => 3,
 	
 	CMD_CONNECT  => 1,
+	CMD_STARTTLS => 10,
 	CMD_EHLO     => 2,
 	CMD_HELO     => 3,
 	CMD_FROM     => 4,
@@ -29,6 +30,7 @@ use constant {
 
 our %CMD = (
 	&CMD_CONNECT  => 'CMD_CONNECT',
+	&CMD_STARTTLS => 'CMD_STARTTLS',
 	&CMD_EHLO     => 'CMD_EHLO',
 	&CMD_HELO     => 'CMD_HELO',
 	&CMD_FROM     => 'CMD_FROM',
@@ -40,7 +42,11 @@ our %CMD = (
 );
 
 has address            => 'localhost';
-has port               => 25;
+has port               => sub { $_[0]->tls ? 465 : 25 };
+has tls                => 0;
+has 'tls_ca';
+has 'tls_cert';
+has 'tls_key';
 has hello              => 'localhost.localdomain';
 has connect_timeout    => sub { $ENV{MOJO_CONNECT_TIMEOUT} || 10 };
 has inactivity_timeout => sub { $ENV{MOJO_INACTIVITY_TIMEOUT} // 20 };
@@ -71,7 +77,7 @@ sub send {
 	
 	# user changed SMTP server or server sent smth while it shouldn't
 	if ($this->{stream} && (($this->{server} ne $this->_server) ||
-	     ($this->{stream}->is_readable && grep {$this->{last_cmd} == $_} (CMD_CONNECT, CMD_DATA_END, CMD_RESET)))
+	     ($this->{stream}->is_readable && !$this->{starttls} && grep {$this->{last_cmd} == $_} (CMD_CONNECT, CMD_DATA_END, CMD_RESET)))
 	) {
 		$this->_rm_stream();
 	}
@@ -80,6 +86,7 @@ sub send {
 		push @steps, sub {
 			my $delay = shift;
 			# connect
+			$this->{starttls} = 0;
 			$this->emit('start');
 			$this->{server} = $this->_server;
 			$this->{last_cmd} = CMD_CONNECT;
@@ -89,9 +96,13 @@ sub send {
 			$this->{client}->on(connect => $connect_cb);
 			$this->{client}->on(error => $connect_cb);
 			$this->{client}->connect(
-				address => $this->address,
-				port    => $this->port,
-				timeout => $this->connect_timeout
+				address  => $this->address,
+				port     => $this->port,
+				timeout  => $this->connect_timeout,
+				tls      => $this->tls,
+				tls_ca   => $this->tls_ca,
+				tls_cert => $this->tls_cert,
+				tls_key  => $this->tls_key
 			);
 		},
 		sub {
@@ -101,25 +112,7 @@ sub send {
 			# check is this a handle
 			Mojo::SMTP::Client::Exception::Stream->throw($_[0]) unless eval { *{$_[0]} };
 			
-			my $error_handler = sub {
-				delete($this->{cleanup_cb})->() if $this->{cleanup_cb};
-				$this->{delay}->remaining([]); # remove remaining steps
-				$this->_rm_stream();
-				$this->{delay}->emit(finish => Mojo::SMTP::Client::Response->new('')->error($_[0]));
-			};
-			
-			$this->{stream} = Mojo::IOLoop::Stream->new($_[0]);
-			$this->{stream}->reactor($this->_ioloop($nb)->reactor);
-			$this->{stream}->start;
-			$this->{stream}->on(timeout => sub {
-				$error_handler->(Mojo::SMTP::Client::Exception::Stream->new('Inactivity timeout'));
-			});
-			$this->{stream}->on(error => sub {
-				$error_handler->(Mojo::SMTP::Client::Exception::Stream->new($_[-1]));
-			});
-			$this->{stream}->on(close => sub {
-				$error_handler->(Mojo::SMTP::Client::Exception::Stream->new('Socket closed unexpectedly by remote side'));
-			});
+			$this->_make_stream($_[0], $this->_ioloop($nb));
 			$this->_read_response($delay->begin, 0);
 			$expected_code = CMD_OK;
 		},
@@ -154,7 +147,75 @@ sub send {
 	for (my $i=0; $i<@cmd; $i+=2) {
 		my $mi = $i+1;
 		
-		if ($cmd[$i] eq 'from') { # FROM
+		if ($cmd[$i] eq 'starttls') { # STARTTLS
+			require IO::Socket::SSL and IO::Socket::SSL->VERSION(0.98);
+			
+			push @steps, sub {
+				my $delay = shift;
+				$this->_cmd('STARTTLS', CMD_STARTTLS);
+				$this->_read_response($delay->begin, $mi == $#cmd);
+				$expected_code = CMD_OK;
+			},
+			$resp_checker,
+			sub {
+				my $delay = shift;
+				$this->{stream}->stop;
+				$this->{stream}->timeout(0);
+				
+				my ($tls_cb, $tid, $loop, $sock);
+				my $error_handler = sub {
+					$loop->remove($tid);
+					$loop->reactor->remove($sock);
+					$sock = undef;
+					$tls_cb->($delay, 0, @_>=2 ? $_[1] : 'Inactivity timeout');
+				};
+				
+				$sock = IO::Socket::SSL->start_SSL(
+					$self->{stream}->steal_handle,
+					SSL_ca_file         => $this->tls_ca,
+					SSL_cert_file       => $this->tls_cert,
+					SSL_key_file        => $this->tls_key,
+					SSL_verify_mode     => $this->tls_ca ? 0x01 : 0x00,
+					SSL_verifycn_name   => $this->address,
+					SSL_verifycn_scheme => $this->tls_ca ? 'smtp' : undef,
+					SSL_startHandshake  => 0,
+					SSL_error_trap      => $error_handler
+				)
+				or return $delay->pass(0, $IO::Socket::SSL::SSL_ERROR);
+				
+				$tls_cb = $delay->begin;
+				$loop = $this->_ioloop($nb);
+				
+				$tid = $loop->timer($this->inactivity_timeout => $error_handler);
+				
+				$loop->reactor->io($sock => sub {
+					if ($sock->connect_SSL) {
+						$loop->remove($tid);
+						$loop->reactor->remove($sock);
+						$this->_make_stream($sock, $loop);
+						$this->{starttls} = 1;
+						$sock = undef;
+						return $tls_cb->($delay, 1);
+					}
+					
+					return $loop->reactor->watch($sock, 1, 0)
+						if $IO::Socket::SSL::SSL_ERROR == IO::Socket::SSL::SSL_WANT_READ;
+					return $loop->reactor->watch($sock, 0, 1)
+						if $IO::Socket::SSL::SSL_ERROR == IO::Socket::SSL::SSL_WANT_WRITE;
+					
+				})->watch($sock, 0, 1);
+			},
+			sub {
+				my ($delay, $success, $error) = @_;
+				unless ($success) {
+					$this->_rm_stream();
+					Mojo::SMTP::Client::Exception::Stream->throw($error);
+				}
+				
+				$delay->pass;
+			}
+		}
+		elsif ($cmd[$i] eq 'from') { # FROM
 			push @steps, sub {
 				my $delay = shift;
 				$this->_cmd('MAIL FROM:<'.$cmd[$mi].'>', CMD_FROM);
@@ -179,7 +240,6 @@ sub send {
 			}
 		}
 		elsif ($cmd[$i] eq 'data') { # DATA
-			# DATA
 			push @steps, sub {
 				my $delay = shift;
 				$this->_cmd('DATA', CMD_DATA);
@@ -263,7 +323,12 @@ sub send {
 	});
 	$delay->on(finish => sub {
 		if ($cb) {
-			$cb->($self, $_[1]);
+			my $r = $_[1];
+			if ($r->isa('Mojo::SMTP::Client::Exception')) {
+				$r = Mojo::SMTP::Client::Response->new('', error => $r);
+			}
+			
+			$cb->($self, $r);
 			$cb = undef;
 			delete $self->{delay};
 		}
@@ -287,7 +352,7 @@ sub _ioloop {
 
 sub _server {
 	my $self = shift;
-	return $self->address.':'.$self->port;
+	return $self->address.':'.$self->port.':'.$self->tls;
 }
 
 sub _cmd {
@@ -296,6 +361,31 @@ sub _cmd {
 	$self->{stream}->write($cmd.Mojo::SMTP::Client::Response::CRLF);
 }
 
+sub _make_stream {
+	my ($self, $sock, $loop) = @_;
+	
+	weaken $self;
+	my $error_handler = sub {
+		delete($self->{cleanup_cb})->() if $self->{cleanup_cb};
+		$self->{delay}->remaining([]); # remove remaining steps
+		$self->_rm_stream();
+		$self->{delay}->emit(finish => $_[0]);
+	};
+	
+	$self->{stream} = Mojo::IOLoop::Stream->new($sock);
+	$self->{stream}->reactor($loop->reactor);
+	$self->{stream}->start;
+	$self->{stream}->on(timeout => sub {
+		$error_handler->(Mojo::SMTP::Client::Exception::Stream->new('Inactivity timeout'));
+	});
+	$self->{stream}->on(error => sub {
+		$error_handler->(Mojo::SMTP::Client::Exception::Stream->new($_[-1]));
+	});
+	$self->{stream}->on(close => sub {
+		$error_handler->(Mojo::SMTP::Client::Exception::Stream->new('Socket closed unexpectedly by remote side'));
+	});
+}
+
 sub _rm_stream {
 	my $self = shift;
 	$self->{stream}->unsubscribe('close')
@@ -438,7 +528,25 @@ Address of SMTP server (ip or domain name). Default is C<localhost>
 
 =head2 port
 
-Port of SMTP server. Default is C<25>
+Port of SMTP server. Default is C<25> for plain connection and C<465> if TLS is enabled.
+
+=head2 tls
+
+Enable TLS. Should be true if SMTP server expects encrypted connection. Default is false.
+Proper version of L<IO::Socket::SSL> should be installed for TLS support in L<Mojo::IOLoop::Client>,
+which you can find with C<mojo version> command.
+
+=head2 tls_ca
+
+Path to TLS certificate authority file. Also activates hostname verification.
+
+=head2 tls_cert
+
+Path to the TLS certificate file.
+
+=head2 tls_key
+
+Path to the TLS key file.
 
 =head2 hello
 
@@ -486,6 +594,15 @@ and C<value> is a value for this command. C<send> understands the following comm
 
 =over
 
+=item starttls
+
+Upgrades connection from plain to encrypted. Some servers requires this before sending any other commands.
+L<IO::Socket::SSL> 0.98+ should be installed for this to work. See also L</tls_ca>, L</tls_cert>, L</tls_key>
+attributes
+
+	$smtp->tls_ca('/etc/ssl/certs/ca-certificates.crt');
+	$smtp->send(starttls => 1);
+
 =item from
 
 From which email this message was sent. Value for this cammand should be a string with email
@@ -559,6 +676,7 @@ clients to make simultaneous sending.
 C<Mojo::SMTP::Client> has this non-importable constants
 
 	CMD_CONNECT  # client connected to SMTP server
+	CMD_STARTTLS # client sent STARTTLS command
 	CMD_EHLO     # client sent EHLO command
 	CMD_HELO     # client sent HELO command
 	CMD_FROM     # client sent MAIL FROM command
